------------------------------------------------------------------------
-- Copyright Notice:                                                    
-- (c) Copyright 2004 - 2008, National Optical Astronomy Observatory, 
-- which is operated for the National Science Foundation by the    
-- Association of Universities for Research in Astronomy under    
-- Cooperative Agreement No. AST-0132798.                                                                                                                           
------------------------------------------------------------------------
-- PROJECT      :   MONSOON-TORRENT
-- BOARD        :   LOCAL CONTROL BOARD
-- DEVICE ID    :   VIRTEX 5LX
-- PARENT MODULE:   SYSTEM TOP LEVEL  
-- MODULE       :   WISHBONE BUS MULTIPLEXOR
-- VERSION      :   2.22
------------------------------------------------------------------------
-- FILE NAME    :   INTERCON.VHD
-- AUTHOR         :   PETER MOORE
-- DATE               :   05/30/2012
-- COMPANY      :   NOAO
-- PLATFORM     :   Xilinx ISE 10.1.03
------------------------------------------------------------------------
-- DESCRIPTION  : 
-- This module acts as the bus multiplexor in distributing data to the
-- major modules in the Torrent firmware architecture. The structure is
-- based upon a Wishbone bus open source architecture and the signals comply
-- to this standard (see http://en.wikipedia.org/wiki/Wishbone_Bus or 
-- http://www.opencores.org/projects.cgi/web/wishbone/wbspec_b3.pdf ). However,
-- in the Torrent firmware design, we use the latent bus (i.e. when it is not
-- being used to transport data) as an asynchronous out of band data path to
-- comunicate the system status to all of the firmware modules in common. In 
-- this respect this firmware deviates from the published specification.
--                 
-- MASTER MUX 
-- Implements a multiplexor to the Wishbone bus between two master devices.
-- Each master posts a Mn_CYC_I signal true to ask for the bus. These signals
-- are arbitrated by the Priority Control logic and a grant to one of the 
-- masters is given. This multiplexor then channels the control signals
-- and data from the Master with the bus grant onto the Wishbone bus.
--
-- SLAVE MUX
-- Provides a multiplexor for control and read data paths
-- for the Wishbone bus architecture. Each master data
-- port provides for selection from any of the slave interfaces
-- or, when not actually used for a read operation, to channel
-- the system status data to each master (Status_in). 
-- 
-- PRIORITY CONTROL
-- This module arbitrates for two Bus Masters who wish to access the Wishbone
-- bus. The arbitration is such that the bus master who last was granted the
-- bus has priority. Arbitration and bus grant is synchronous to the bus clock.                 
------------------------------------------------------------------------
-- IMPLEMENTATION NOTES :
-- MASTER MUX
--  This data path logic is purely combinational i.e. there is no clock synchronization 
--  used in an effort to reduce the bus latency. 
--
-- SLAVE MUX               
--  When multiple slaves are selected by the STB_I address
--  (Hopefully only for writes !) then the acknowledge logic
--  will react to multiple ack (or err, or rty) signals and
--  only pass on the control signal when all slaves have 
--  acknowledged. Note that this scheme doesn't currently 
--  support any method to supply correct acknowledge signals
--  when one or more slave interfaces posts an error or retry
--  signal. Under these conditions a bus lockup occurs which 
--  must be resolved by the master bus timeout function.
--
--  The status assignments are common fixed assignments in this
--  file. Use this space to select which status bits from each
--  slave interface are transmitted to the complete system.
--
-- PRIORITY CONTROL
------------------------------------------------------------------------
-- REVISION HISTORY :
-- DATE	  AUTHOR  VERSION  MODIFICATIONS
-- 10/05/2008  PCM   1.00   ORIGINAL VERSION
-- 10/29/2008  PCM   1.01   TOOK ORIGINAL MODULES AND COMBINED THESE INTO ONE FILE. IT'S A LITTLE LESS WORK THIS WAY
-- 5/25/2009   PCM   1.02   REMOVED RESET TIMER LOGIC. THIS IS NOW CONTROLLED BY THE LCB_CONTROL MODULE. ADDED BROADCAST ACKNOWLEDGE LOGIC
--                                              FOR RESET AND START EXPOSURE COMMANDS. ADDED STATUS ASSIGNMENTS.
-- 9/13/2009    PCM   1.03   ADDED DUMMY RESPONSE FOR READS WITH MODULE SELECT VALUES OF ZERO.
-- 02/10/2010   PCM   1.04   ADDED DEBUG PORT, ADDDED STATUS CONNECTIONS FOR ShutDownQuick
-- 02/22/2010  PCM   1.05   Added extra three ports and setup the system version number 
-- 03/11/2010    PCM   1.06   Changed master arbitration to allow one clock cycle pause between switching from one master to the other. Data is corrupted when a master change occurs immediately after 
--                                              a write by the previous master.
-- 05/05/2010   PCM   1.18  Changed system version number and synchronized the intercon version number with the system version build number
-- 06/08/2010   PCM   1.19  Added PSM AFE_PWRON status linkage
-- 06/21/2010   PCM    2.00  Just a version number change
-- 09/30/2010  PCM    2.01   Just a version number change
-- 11/23/2010  PCM    2.02   Just a version number change
-- 01/25/2011  PCM    2.03   Just a version number change
-- 02/02/2011  PCM    2.04   Just a version number change
-- 03/25/2011  PCM    2.05   Just a version number change
-- 04/14/2011  PCM    2.06   Just a version number change
-- 04/31/2011  PCM    2.07   Just a version number change
-- 06/16/2011  PCM    2.08   Added PSM status bit 6 to be bit 23 of global status word (VHVPolarity bit)
-- 10/02/2011  PCM    2.10   Just a version number change
-- 02/27/2012  PCM   2.11   Just a version number change
-- 03/05/2012  PCM   2.20   Just a version number change
-- 03/21/2012  PCM   2.21   Just a version number change
-- 05/30/2012  PCM   2.22   Just a version number change
--
------------------------------------------------------------------------
-- PORT DEFINITIONS :
-- M0_ADR_I       : 16-bit 'module' Address from Master 0 (highest priority)
-- M0_CYC_I       : Master 0 Cycle request
-- M0_DATA_I      : 32-bit write data from Master 0 
-- M0_GRNT        : Master 0 Cycle grant from priority logic
-- M0_STB_I       : 8-bit Master 0 'module' Select Strobes 
-- M0_TGC_I       : Master 0 Cycle Type tag identifier input
-- M0_TGD_I       : Master 0 Data Type tag identifier input
-- M0_WE_I        : Master 0 Write request input strobe
-- M1_ADR_I       : 16-bit 'module' Address from Master 1
-- M1_CYC_I       : Master 1 Cycle request
-- M1_DATA_I      : 32-bit write data from Master 1 
-- M1_GRNT        : Master 1 Cycle grant from priority  logic
-- M1_STB_I       : 8-bit Master 1 'module' Select Strobes
-- M1_TGC_I       : Master 1 Cycle Type tag identifier input
-- M1_TGD_I       : Master 1 Data Type tag identifier input
-- M1_WE_I        : Master 1 Write request input strobe
-- SLAVE_ADR_O    : Wishbone Address bus
-- SLAVE_CYC_O    : Wishbone Cycle strobe output 
-- SLAVE_DATA_O   : Wishbone Data bus
-- SLAVE_STB_O    : Wishbone individual 'Module' select strobes 
-- SLAVE_TGC_O    : Wishbone Cycle Type tag
-- SLAVE_TGD_O    : Wishbone Data Type tag 
-- SLAVE_WE_O     : Wishbone Write Enable strobe.
-- CLK_I          : Bus clock input - normally the system clock - one that never stops !
-- CLK_O          : Bus clock output
-- RST_O          : Bus reset pulse - once on power up and then from RST_I trigger
-- RST_I          : external reset signal in. Distributed to all bus interface logic modules.
-- M0_ACK_O       : Normal bus cycle termination acknowledge strobe from slave interface(s) to Master 0
-- M0_ERR_O       : Error bus cycle termination acknowledge strobe from slave interface(s) to Master 0    
-- M0_RTY_O       : Retry bus cycle termination acknowledge strobe from slave interface(s) to Master 0   
-- M0_TGD_O       : Data tag value from Slave interface to Master 0
-- M0_DATA_O      : Read data from slave interface to Master 0
-- M1_ACK_O       : Normal bus cycle termination acknowledge strobe from slave interface(s) to Master 1   
-- M1_ERR_O       : Error bus cycle termination acknowledge strobe from slave interface(s) to Master 1    
-- M1_RTY_O       : Retry bus cycle termination acknowledge strobe from slave interface(s) to Master 1    
-- M1_TGD_O       : Data tag value from Slave interface to Master 1   
-- M1_DATA_O      : Read data from slave interface to Master 1   
-- SLAVE_ACK_I    : Inputs from Slave interfaces for Normal bus cycle termination
-- SLAVE_ERR_I    : Inputs from Slave interfaces for Error bus cycle termination
-- SLAVE_RTY_I    : Inputs from Slave interfaces for Retry bus cycle termination
-- SLAVE_TGD_I    : Inputs from Slave interfaces for read data tag information
-- S0_DATA_I      : Read (input) data from Slave interface 0
-- S1_DATA_I      : Read (input) data from Slave interface 1  
-- S2_DATA_I      : Read (input) data from Slave interface 2  
-- S3_DATA_I      : Read (input) data from Slave interface 3  
-- S4_DATA_I      : Read (input) data from Slave interface 4
------------------------------------------------------------------------
--  System Attributes Go Here !!
------------------------------------------------------------------------
-- #MD:SYS:0xFF:    --This is the Wishbone module select address for system level commands
-- #AD: NONE: SysRebootCmd: 1: 1: A: SYS: 0xFFFF: 0: 1: SMPL:  NONE: CMND: 20: : Reboot:Triggers a Firmware Reboot
-- #AD: NONE: SysResetCmd: 1: 1: A:  SYS: 0xFFFE: 0: 1:  SMPL: NONE: CMND: 20: : Reset: Triggers a system level reset
-- #AD: NONE: SysCodeId: 1: 1: A:  SYS: 0xFFFF: 0: 16:  NONE: SMPL: CNST: 20: 2.04: Version: System level build code as version MM.mm

library ieee;
use ieee.std_logic_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Intercon_Ver222 is
   port ( CLK_I        : in    std_logic; 
          M0_ADR_I     : in    std_logic_vector (15 downto 0); 
          M0_CYC_I     : in    std_logic; 
          M0_DATA_I    : in    std_logic_vector (31 downto 0); 
          M0_STB_I     : in    std_logic_vector (7 downto 0); 
          M0_TGC_I     : in    std_logic_vector (1 downto 0); 
          M0_TGD_I     : in    std_logic_vector(3 downto 0); 
          M0_WE_I      : in    std_logic; 
          M1_ADR_I     : in    std_logic_vector (15 downto 0); 
          M1_CYC_I     : in    std_logic; 
          M1_DATA_I    : in    std_logic_vector (31 downto 0); 
          M1_STB_I     : in    std_logic_vector (7 downto 0); 
          M1_TGC_I     : in    std_logic_vector (1 downto 0); 
          M1_TGD_I     : in    std_logic_vector(3 downto 0); 
          M1_WE_I      : in    std_logic; 
          RST_I        : in    std_logic; 
          SLAVE_ACK_I  : in    std_logic_vector (7 downto 0); 
          SLAVE_ERR_I  : in    std_logic_vector (7 downto 0); 
          SLAVE_RTY_I  : in    std_logic_vector (7 downto 0); 
          S0_TGD_I     : in    std_logic_vector (3 downto 0); 
          S1_TGD_I     : in    std_logic_vector (3 downto 0); 
          S2_TGD_I     : in    std_logic_vector (3 downto 0); 
          S3_TGD_I     : in    std_logic_vector (3 downto 0); 
          S4_TGD_I     : in    std_logic_vector (3 downto 0); 
          S5_TGD_I     : in    std_logic_vector (3 downto 0); 
          S6_TGD_I     : in    std_logic_vector (3 downto 0); 
          S7_TGD_I     : in    std_logic_vector (3 downto 0); 
          S0_DATA_I    : in    std_logic_vector (31 downto 0); 
          S1_DATA_I    : in    std_logic_vector (31 downto 0); 
          S2_DATA_I    : in    std_logic_vector (31 downto 0); 
          S3_DATA_I    : in    std_logic_vector (31 downto 0); 
          S4_DATA_I    : in    std_logic_vector (31 downto 0); 
          S5_DATA_I    : in    std_logic_vector (31 downto 0); 
          S6_DATA_I    : in    std_logic_vector (31 downto 0); 
          S7_DATA_I    : in    std_logic_vector (31 downto 0); 
          CLK_O        : out   std_logic; 
          MSTR0_ACK_O  : out   std_logic; 
          MSTR0_DATA_O : out   std_logic_vector (31 downto 0); 
          MSTR0_ERR_O  : out   std_logic; 
          MSTR0_RTY_O  : out   std_logic; 
          MSTR0_TGD_O  : out   std_logic_vector(3 downto 0); 
          MSTR1_ACK_O  : out   std_logic; 
          MSTR1_DATA_O : out   std_logic_vector (31 downto 0); 
          MSTR1_ERR_O  : out   std_logic; 
          MSTR1_RTY_O  : out   std_logic; 
          MSTR1_TGD_O  : out   std_logic_vector(3 downto 0); 
          M0_CYC_GRNT  : out   std_logic; 
          M1_CYC_GRNT  : out   std_logic; 
          RST_O        : out   std_logic; 
          SLAVE_ADR_O  : out   std_logic_vector (15 downto 0); 
          SLAVE_CYC_O  : out   std_logic; 
          SLAVE_DATA_O : out   std_logic_vector (31 downto 0); 
          SLAVE_STB_O  : out   std_logic_vector (7 downto 0); 
          SLAVE_TGC_O  : out   std_logic_vector (1 downto 0); 
          SLAVE_TGD_O  : out   std_logic_vector(3 downto 0); 
          SLAVE_WE_O   : out   std_logic;
          SysDbg       : out   std_logic_vector(7 downto 0)
          );
end Intercon_Ver222;

architecture Behavioral of Intercon_Ver222 is
-- Version numbers
   constant FIRMWARE_BUILD_VERSION : std_logic_vector(31 downto 0) := x"000000DE";   -- Current version number is 2.22

--  Master Mux Segment --
   signal WrGrant0, WrGrant1, Mstr0VerReq, Mstr1VerReq : std_logic;

-- Slave Mux Segment --
   signal SlaveAck0, SlaveAck1, SlaveError, SlaveErr0, SlaveErr1, SlaveRty0, SlaveRty1 : std_logic;
   signal StatReg_Lcb, StatReg_Psm, StatReg_Cfg, StatReg_Pix : std_logic_vector(7 downto 0);
   signal StatReg_Afe, StatReg_Clk, StatReg_Spare_1, StatReg_Spare_2 : std_logic_vector(7 downto 0);
   signal StatEnb0, StatEnb1, StatEnb2, StatEnb3, StatEnb4, StatEnb5, StatEnb6, StatEnb7 : std_logic;
   signal StatReg_dummy : std_logic := '0';
   signal BroadcastAck_0, BroadcastAck_1 : std_logic;
   
   signal Status_Out: std_logic_vector(31 downto 0);

-- Arbitration Logic Segment --
   signal Mstr0Grnt, Mstr1Grnt, Mstr0PreGrant, Mstr1PreGrant : std_logic;   
   signal slaveCycle, SlaveCycleEnable, slaveWrite, slaveWrtReq : std_logic;

begin

---------------------------------------------------------------------------------------------------------------
--  Debug Port --
---------------------------------------------------------------------------------------------------------------  

   SysDbg(7) <= Mstr1Grnt;
   SysDbg(6) <= Mstr0Grnt;
   SysDbg(5) <= slaveCycle;
   SysDbg(4) <= slaveWrite;
   SysDbg(3) <= SlaveError;
   SysDbg(2) <= not(WrGrant0 or WrGrant1);  -- Status enable to slaves
   SysDbg(1) <= SlaveAck1;
   SysDbg(0) <= SlaveAck0;
   
   SlaveError   <= SlaveErr0 or SlaveErr1;

---------------------------------------------------------------------------------------------------------------
   --  Priority / Arbitration Segment --
--------------------------------------------------------------------------------------------------------------- 
   M0_CYC_GRNT <= Mstr0Grnt;
   M1_CYC_GRNT <= Mstr1Grnt;
   
   BusArbiterProc : process(CLK_I)
   begin
      if (rising_edge(CLK_I)) then
         Mstr0PreGrant <= '0';               -- Kill any leftovers from a previous bus master change after this clock cycle
         Mstr1PreGrant <= '0';
         SlaveCycleEnable <= '1';
         if (RST_I = '1') then
            Mstr0Grnt <= '0';                -- Give priority to first master to request after a reset.
            Mstr1Grnt <= '0';
         elsif (slaveCycle = '0') then       -- Only arbitrate when the no bus activity
            if (Mstr0PreGrant = '1' and M0_CYC_I = '1') then     -- We are here because M0 requested the bus last cycle, is it still requesting ?
               Mstr0Grnt <= '1';
            elsif (Mstr1PreGrant = '1' and M1_CYC_I = '1') then  -- We are here because M1 requested the bus last cycle, is it still requesting ?
               Mstr1Grnt <= '1';
            elsif (Mstr0Grnt = '1' and M1_CYC_I = '1') then      -- Currently, M0 has the bus. See if we have a request for M1 pending
               Mstr0Grnt <= '0';                                 -- Pause the proceedings and prepare for a bus master change
               Mstr1PreGrant <= '1';                             -- Change grant on the next clock cycle
            elsif (Mstr1Grnt = '1' and M0_CYC_I = '1') then      -- Look at M0 if we have no pending request from M0
               Mstr1Grnt <= '0';
               Mstr0PreGrant <= '1';
            elsif (M0_CYC_I = '1') then                          -- Here if we had a PreGrant active but the master decided to abandon us or a reset event
               Mstr0Grnt <= '1';
            elsif (M1_CYC_I = '1') then
               Mstr1Grnt <= '1';
            else
               SlaveCycleEnable <= '0';                        -- Here someone already has control so we delay the CYC strobe to coincide with the other slave control signals controlled by the current master
           end if;
         end if;
      end if;
   end process;

---------------------------------------------------------------------------------------------------------------
   --  Master Mux Segment --
--------------------------------------------------------------------------------------------------------------- 
-- Slave address and control multiplexing --
   slaveCycle  <= M0_CYC_I when (Mstr0Grnt = '1') else
                  M1_CYC_I when (Mstr1Grnt = '1') else '0';
   SLAVE_STB_O <= M0_STB_I when (Mstr0Grnt = '1') else 
                  M1_STB_I when (Mstr1Grnt = '1') else (others => '0');
   SLAVE_TGC_O <= M0_TGC_I when (Mstr0Grnt = '1') else 
                  M1_TGC_I when (Mstr1Grnt = '1') else (others => '0');
   SLAVE_TGD_O <= M0_TGD_I when (Mstr0Grnt = '1') else
                  M1_TGD_I when (Mstr1Grnt = '1') else (others => '0'); 
   slaveWrtReq <= M0_WE_I  when (Mstr0Grnt = '1') else
                  M1_WE_I  when (Mstr1Grnt = '1') else '0';
   SLAVE_ADR_O <= M0_ADR_I when (Mstr0Grnt = '1') else M1_ADR_I;
                  
   slaveWrite  <= slaveWrtReq and slaveCycle;
   SLAVE_WE_O  <= slaveWrite;
   SLAVE_CYC_O <= slaveCycle and SlaveCycleEnable;
   
-- Slave data pathway multiplexing --   
   WrGrant0 <= Mstr0Grnt and M0_WE_I;
   WrGrant1 <= Mstr1Grnt and M1_WE_I;
   SLAVE_DATA_O <= M0_DATA_I when (WrGrant0 = '1') else M1_DATA_I when (WrGrant1 = '1') else Status_Out; 

-- Broadcast commands (Start exposure, Reset) acknowledge strobe and catch for when module address equals zero
   BroadCastAckProc : process (CLK_I)
   begin
      if (rising_edge(CLK_I)) then
         BroadcastAck_0 <= '0';
         BroadcastAck_1 <= '0';
         if (WrGrant0 = '1' and M0_CYC_I = '1') then
            if ((M0_TGC_I /= "00" and M0_STB_I = "11111111") or M0_STB_I = "00000000") then
               BroadcastAck_0 <= '1';
            end if;
         elsif (Mstr1Grnt = '1' and M1_CYC_I = '1') then
            if ((M1_TGC_I /= "00" and M1_STB_I = "11111111") or M1_STB_I = "00000000") then
               BroadcastAck_1 <= '1';
            end if;
         end if;
      end if;
   end process;
   
   VersionReqProc : process(CLK_I)
   begin
      if (rising_edge(CLK_I)) then
         if (M0_CYC_I = '1' and M0_TGC_I = "00" and M0_STB_I = "11111111" and M0_ADR_I = "1111111111111111" and M0_WE_I = '0') then
            Mstr0VerReq <= '1';        -- Force the version data onto the master read port
         elsif (M1_CYC_I = '1' and M1_TGC_I = "00" and M1_STB_I = "11111111" and M1_ADR_I = "1111111111111111" and M1_WE_I = '0') then
            Mstr1VerReq <= '1';        -- Force the version data onto the master read port
         elsif (slaveCycle = '0') then
            Mstr0VerReq <= '0';        -- Clear the flags after the current cycle
            Mstr1VerReq <= '0';
         end if;
      end if;
   end process;
   
---------------------------------------------------------------------------------------------------------------
   --  Slave Mux Segment --
---------------------------------------------------------------------------------------------------------------  
-- Clock distribution --
   CLK_O <= CLK_I;
 
-- Control signal development --
   SlaveAck0 <= (((SLAVE_ACK_I(0) or not M0_STB_I(0)) and (SLAVE_ACK_I(1) or not M0_STB_I(1)) and
                (SLAVE_ACK_I(2) or not M0_STB_I(2)) and (SLAVE_ACK_I(3) or not M0_STB_I(3)) and
                (SLAVE_ACK_I(4) or not M0_STB_I(4)) and (SLAVE_ACK_I(5) or not M0_STB_I(5)) and 
                (SLAVE_ACK_I(6) or not M0_STB_I(6)) and (SLAVE_ACK_I(7) or not M0_STB_I(7))) or BroadcastAck_0) and M0_CYC_I and Mstr0Grnt and SlaveCycleEnable;

   SlaveAck1 <= (((SLAVE_ACK_I(0) or not M1_STB_I(0)) and (SLAVE_ACK_I(1) or not M1_STB_I(1)) and
                (SLAVE_ACK_I(2) or not M1_STB_I(2)) and (SLAVE_ACK_I(3) or not M1_STB_I(3)) and
                (SLAVE_ACK_I(4) or not M1_STB_I(4)) and (SLAVE_ACK_I(5) or not M1_STB_I(5)) and 
                (SLAVE_ACK_I(6) or not M1_STB_I(6)) and (SLAVE_ACK_I(7) or not M1_STB_I(7))) or BroadcastAck_1) and M1_CYC_I and Mstr1Grnt and SlaveCycleEnable;
                
   SlaveErr0 <= ((SLAVE_ERR_I(0) and M0_STB_I(0)) or (SLAVE_ERR_I(1) and M0_STB_I(1)) or
                 (SLAVE_ERR_I(2) and M0_STB_I(2)) or (SLAVE_ERR_I(3) and M0_STB_I(3)) or
                 (SLAVE_ERR_I(4) and M0_STB_I(4)) or (SLAVE_ERR_I(5) and M0_STB_I(5)) or
                 (SLAVE_ERR_I(6) and M0_STB_I(6)) or (SLAVE_ERR_I(7) and M0_STB_I(7))) and M0_CYC_I and Mstr0Grnt and SlaveCycleEnable;

   SlaveErr1 <= ((SLAVE_ERR_I(0) and M1_STB_I(0)) or (SLAVE_ERR_I(1) and M1_STB_I(1)) or 
                 (SLAVE_ERR_I(2) and M1_STB_I(2)) or (SLAVE_ERR_I(3) and M1_STB_I(3)) or 
                 (SLAVE_ERR_I(4) and M1_STB_I(4)) or (SLAVE_ERR_I(5) and M1_STB_I(5)) or 
                 (SLAVE_ERR_I(6) and M1_STB_I(6)) or (SLAVE_ERR_I(7) and M1_STB_I(7))) and M1_CYC_I and Mstr1Grnt and SlaveCycleEnable;
                
   SlaveRty0 <= ((SLAVE_RTY_I(0) and M0_STB_I(0)) or (SLAVE_RTY_I(1) and M0_STB_I(1)) or 
                 (SLAVE_RTY_I(2) and M0_STB_I(2)) or (SLAVE_RTY_I(3) and M0_STB_I(3)) or 
                 (SLAVE_RTY_I(4) and M0_STB_I(4)) or (SLAVE_RTY_I(5) and M0_STB_I(5)) or 
                 (SLAVE_RTY_I(6) and M0_STB_I(6)) or (SLAVE_RTY_I(7) and M0_STB_I(7))) and M0_CYC_I and Mstr0Grnt and SlaveCycleEnable;

   SlaveRty1 <= ((SLAVE_RTY_I(0) and M1_STB_I(0)) or (SLAVE_RTY_I(1) and M1_STB_I(1)) or 
                 (SLAVE_RTY_I(2) and M1_STB_I(2)) or (SLAVE_RTY_I(3) and M1_STB_I(3)) or 
                 (SLAVE_RTY_I(4) and M1_STB_I(4)) or (SLAVE_RTY_I(5) and M1_STB_I(5)) or 
                 (SLAVE_RTY_I(6) and M1_STB_I(6)) or (SLAVE_RTY_I(7) and M1_STB_I(7))) and M1_CYC_I and Mstr1Grnt and SlaveCycleEnable;
                
   MSTR0_ACK_O <= '0' when (Mstr0Grnt = '0') else SlaveAck0;
   MSTR1_ACK_O <= '0' when (Mstr1Grnt = '0') else SlaveAck1;
   MSTR0_ERR_O <= '0' when (Mstr0Grnt = '0') else SlaveErr0;
   MSTR1_ERR_O <= '0' when (Mstr1Grnt = '0') else SlaveErr1;
   MSTR0_RTY_O <= '0' when (Mstr0Grnt = '0') else SlaveRty0;
   MSTR1_RTY_O <= '0' when (Mstr1Grnt = '0') else SlaveRty1;
   
-- Data path mux --
   MSTR0_DATA_O <= FIRMWARE_BUILD_VERSION when (Mstr0VerReq = '1') else
                S0_DATA_I when (M0_STB_I(0) = '1') else
                S1_DATA_I when (M0_STB_I(1) = '1') else
                S2_DATA_I when (M0_STB_I(2) = '1') else
                S3_DATA_I when (M0_STB_I(3) = '1') else
                S4_DATA_I when (M0_STB_I(4) = '1') else
                S5_DATA_I when (M0_STB_I(5) = '1') else
                S6_DATA_I when (M0_STB_I(6) = '1') else
                S7_DATA_I when (M0_STB_I(7) = '1') else
                (others => '0');
                
   MSTR1_DATA_O <= FIRMWARE_BUILD_VERSION when (Mstr1VerReq = '1') else
                S0_DATA_I when (M1_STB_I(0) = '1') else
                S1_DATA_I when (M1_STB_I(1) = '1') else
                S2_DATA_I when (M1_STB_I(2) = '1') else
                S3_DATA_I when (M1_STB_I(3) = '1') else
                S4_DATA_I when (M1_STB_I(4) = '1') else
                S5_DATA_I when (M1_STB_I(5) = '1') else
                S6_DATA_I when (M1_STB_I(6) = '1') else
                S7_DATA_I when (M1_STB_I(7) = '1') else
                (others => '0');
                
   MSTR0_TGD_O  <= M0_TGD_I when (Mstr0VerReq = '1') else
                S0_TGD_I when (M0_STB_I(0) = '1') else
                S1_TGD_I when (M0_STB_I(1) = '1') else
                S2_TGD_I when (M0_STB_I(2) = '1') else
                S3_TGD_I when (M0_STB_I(3) = '1') else
                S4_TGD_I when (M0_STB_I(4) = '1') else
                S5_TGD_I when (M0_STB_I(5) = '1') else
                S6_TGD_I when (M0_STB_I(6) = '1') else
                S7_TGD_I when (M0_STB_I(7) = '1') else
                (others =>'0');

   MSTR1_TGD_O  <= M1_TGD_I when (Mstr0VerReq = '1') else
                S0_TGD_I when (M1_STB_I(0) = '1') else
                S1_TGD_I when (M1_STB_I(1) = '1') else
                S2_TGD_I when (M1_STB_I(2) = '1') else
                S3_TGD_I when (M1_STB_I(3) = '1') else
                S4_TGD_I when (M1_STB_I(4) = '1') else
                S5_TGD_I when (M1_STB_I(5) = '1') else
                S6_TGD_I when (M1_STB_I(6) = '1') else
                S7_TGD_I when (M1_STB_I(7) = '1') else
                (others =>'0');

-- Status assignments -- Note: Add to these assignments using the same Intercon version but if you change a status bit assignment, Change the Intercon version !!!
   Status_out(0) <=  StatReg_Psm(0);    -- TempChanSlct(0) 
   Status_out(1) <=  StatReg_Psm(1);    -- TempChanSlct(1) 
   Status_out(2) <=  StatReg_Psm(2);    -- TempChanSlct(2 )
   Status_out(3) <=  StatReg_Cfg(1);    -- AFE1_Detected
   Status_out(4) <=  StatReg_Cfg(2);    -- AFE2_Detected
   Status_out(5) <=  StatReg_Pix(0);    -- PackedPixels
   Status_out(6) <=  StatReg_Pix(1);    -- Pixels16
   Status_out(7) <=  StatReg_Cfg(3);    -- DheIsSlave
   Status_out(8) <=  StatReg_Pix(2);    -- StreamDataMode
   Status_out(9) <=  StatReg_dummy ;    --
   Status_out(10) <= StatReg_dummy ;    --
   Status_out(11) <= StatReg_dummy ;    --
   Status_out(12) <= StatReg_dummy ;    --
   Status_out(13) <= StatReg_dummy ;    --
   Status_out(14) <= StatReg_dummy ;    --
   Status_out(15) <= StatReg_dummy ;    --
   Status_out(16) <= StatReg_dummy ;    --
   Status_out(17) <= StatReg_dummy ;    --
   Status_out(18) <= StatReg_dummy ;    --
   Status_out(19) <= StatReg_dummy ;    --
   Status_out(20) <= StatReg_dummy ;    --
   Status_out(21) <= StatReg_Afe(0);    -- Afe 1 is enabled
   Status_out(22) <= StatReg_Afe(1);    -- Afe 2 is enabled
   Status_out(23) <= StatReg_Psm(6);    -- Vhv polarity is positive (N-Channel mode) when true
   Status_out(24) <= StatReg_Psm(5);    -- AFEs have power on them
   Status_out(25) <= StatReg_Clk(0);    -- Clocks are locked
   Status_out(26) <= StatReg_Psm(4);    -- Memory Power Available
   Status_out(27) <= StatReg_Cfg(5);    -- Line Start Flag
   Status_out(28) <= StatReg_Cfg(4);    -- Frame Start Flag
   Status_out(29) <= StatReg_Cfg(0);    -- ReadoutInProgress
   Status_out(30) <= StatReg_Lcb(0);    -- AsyncFlag
   Status_out(31) <= StatReg_Cfg(3);    -- ShutDownQuick
   
-- Status latches --
   StatEnb0 <= not (M0_STB_I(0) or M1_STB_I(0)) or (M0_STB_I(0) and WrGrant0) or (M1_STB_I(0) and WrGrant1);
   StatEnb1 <= not (M0_STB_I(1) or M1_STB_I(1)) or (M0_STB_I(1) and WrGrant0) or (M1_STB_I(1) and WrGrant1);
   StatEnb2 <= not (M0_STB_I(2) or M1_STB_I(2)) or (M0_STB_I(2) and WrGrant0) or (M1_STB_I(2) and WrGrant1);
   StatEnb3 <= not (M0_STB_I(3) or M1_STB_I(3)) or (M0_STB_I(3) and WrGrant0) or (M1_STB_I(3) and WrGrant1);
   StatEnb4 <= not (M0_STB_I(4) or M1_STB_I(4)) or (M0_STB_I(4) and WrGrant0) or (M1_STB_I(4) and WrGrant1);
   StatEnb5 <= not (M0_STB_I(5) or M1_STB_I(5)) or (M0_STB_I(5) and WrGrant0) or (M1_STB_I(5) and WrGrant1);
   StatEnb6 <= not (M0_STB_I(6) or M1_STB_I(6)) or (M0_STB_I(6) and WrGrant0) or (M1_STB_I(6) and WrGrant1);
   StatEnb7 <= not (M0_STB_I(7) or M1_STB_I(7)) or (M0_STB_I(7) and WrGrant0) or (M1_STB_I(7) and WrGrant1);
   
   StatRegProc0 : process(CLK_I)
   begin
      if (rising_edge(CLK_I) and StatEnb0 = '1') then
         StatReg_Lcb <= S0_DATA_I(7 downto 0);
      end if;
   end process;
   
   StatRegProc1 : process(CLK_I)
   begin
      if (rising_edge(CLK_I) and StatEnb1 = '1') then
         StatReg_Psm <= S1_DATA_I(7 downto 0);
      end if;
   end process;

   StatRegProc2 : process(CLK_I)
   begin
      if (rising_edge(CLK_I) and StatEnb2 = '1') then
         StatReg_Cfg <= S2_DATA_I(7 downto 0);
      end if;
   end process;

   StatRegProc3 : process(CLK_I)
   begin
      if (rising_edge(CLK_I) and StatEnb3 = '1') then
         StatReg_Pix <= S3_DATA_I(7 downto 0);
      end if;
   end process;

   StatRegProc4 : process(CLK_I)
   begin
      if (rising_edge(CLK_I) and StatEnb4 = '1') then
         StatReg_Afe <= S4_DATA_I(7 downto 0);
      end if;
   end process;

   StatRegProc5 : process(CLK_I)
   begin
      if (rising_edge(CLK_I) and StatEnb5 = '1') then
         StatReg_Spare_1 <= S5_DATA_I(7 downto 0);
      end if;
   end process;
   
   StatRegProc6 : process(CLK_I)
   begin
      if (rising_edge(CLK_I) and StatEnb6 = '1') then
         StatReg_Spare_2 <= S6_DATA_I(7 downto 0);
      end if;
   end process;
   
   StatRegProc7 : process(CLK_I)
   begin
      if (rising_edge(CLK_I) and StatEnb7 = '1') then
         StatReg_Clk <= S7_DATA_I(7 downto 0);
      end if;
   end process;
     
---------------------------------------------------------------------------------------------------------------
   --  Reset logic Segment --
--------------------------------------------------------------------------------------------------------------- 

    RST_O <= RST_I;
   
 end BEHAVIORAL;


